#define MAIN_PROGRAM
#define	BP_DEBUG
#include <stdio.h>
#include <modes.h>
#include <balboa.h>
#include <bp_error.h>
#include <bp_mem.h>
#include <vm_video.h>
#include <vm_dcp.h>
#include <vm_vs.h>
#include <vm_txt.h>
#include <sgm.h>
#include <controls.h>
#include <bam.h>
#include <ucm.h>
#include <vm_clut.h>
#include <cdfm.h>
#include <csd.h>
#include <signal.h>
#include <iff.h>
#include <iff_aiff.h>

VS		*VideoScreen;
PICTURE	*Picture;
HOTSPOT	*HotSpot;
int		interlace_setting;
int		vres_setting;

void exit_balboa();
void force_exit();

int init_balboa(argc, argv)
	int		argc;
	char	*argv[];
{
	sgm_enable(SIGINT, force_exit, 0, IMMEDIATE);
	sgm_enable(SIGQUIT, force_exit, 0, IMMEDIATE);

    dispatch_atquit(exit_balboa, NULL);

	STATUS_INIT(stdout, ST_MGR_ALL & ~(ST_TMR_MGR | ST_ANM_MGR), ST_TYP_ALL & ~ST_ITERATE, 50);

	init_video("Title.iff");

	printf("Running!\n");

    return	OK;
}

void FIFOFullFunc()
{
    STATUS(ST_APPL, ST_INFO_STR, 0, "Dispatcher FIFO is 90% full");
}

int main(argc, argv)
	int		argc;
	char	*argv[];
{
    interlace_setting	= 0;
    vres_setting		= 0;
	errno				= 0;

    sgm_init();

	dispatch_init(75, BP_MEM_DONTCARE, 60, FIFOFullFunc);
	dispatch_loop(init_balboa, argc, argv);
}

void force_exit(sig, param)
	short sig;
	int param;
{
    dispatch_quit(errno);
}

void exit_balboa()
{
	/* Shutdown video & Cursor */
	vs_close(VideoScreen);
    vs_finish();

	if (errno != 0)
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "Balboa: Bad Exit");
	}

	else
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_INFO_STR, 0, "Balboa: Good Exit");
	}

    exit(0);
}

int init_video(filename)
    register char  *filename;
{
	/* Initialize the video manager with two video large buffers */
	if (vs_init(BP_MEM_PLANEA, 2) == SYSERR)
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: vs_init() failed");

		dispatch_quit(SYSERR);
	}

	/* Open a video screen for LCT double buffering and Cursor On */
	if (!(VideoScreen = vs_open(LCT_DOUBLE, 3, BP_MEM_PLANEB)))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: vs_open() failed");

		dispatch_quit(SYSERR);
	}

	/* Initialize the CLUT manager for this Video Screen */
	if (cl_init(VideoScreen, BP_MEM_DONTCARE, 0) == SYSERR)
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: cl_init() failed to initialize CLUT manager");

		dispatch_quit(SYSERR);
	}

	/* Input our IFF image file */
	if (!(Picture = pi_iffpic(PLANE_A, vres_setting, filename, BP_MEM_DONTCARE)))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: pi_iffpic() failed to load file");

		dispatch_quit(SYSERR);
	}

	/* Take care of any associated CLUT */
	if (Picture->pi_clut)
	{
		if (SYSERR == cl_exec(VideoScreen, PLANE_A, Picture->pi_clut))
		{
			STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
				"init_video: Unable to write picture's CLUT");

			dispatch_quit(SYSERR);
		}
    }

	/* Place our picture at the front */
	if (SYSERR == pi_front(VideoScreen, Picture))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
			"init_video: pi_front() failed to place picture");

		dispatch_quit(SYSERR);
	}

	/* Update the LCTs for this picture */
	if (SYSERR == vs_update(VideoScreen, interlace_setting, vm_display_type, NULL, NULL))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
			"init_video: vs_update() failed to update LCTs");

		dispatch_quit(SYSERR);
	}

	/* Display the Video Screen (Activate FCTs and cursor) */
	if (SYSERR == vs_show(VideoScreen))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: vs_show() failed");

		dispatch_quit(SYSERR);
	}

	return	OK;
}

